//
// Simple Bang-Bang thermostat control, using Velleman K8055 controller board
// and the RoboCooler.
//
// This is free software and is offered with no warranty expressed or implied.
//
// k8055thermo.c : Defines the entry point for the console application.
//

#include <stdio.h>
#include "k8055_fixup.h" // Wrappers to expose k8055.dll functions for C language caller
#include <stdlib.h>
#include <unistd.h>    // standard unix functions, like getpid()      /
#include <signal.h>    // signal name macros, and the signal() prototype */
#include <math.h>

#include <k8055.h>

#include "ktime.h"

// variables needed for graph
int sum = 0;
int i, j;
static int values[10] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
int average;
char smooth_rough;

static FILE* logFilePtr ;

// Global variables
int gSignalCaught = 0 ;		// Set to true by signal handler to force exit.
int gMode = IDLE ; 
unsigned long gSeq = 0L ;
int gFanRunTime = 0 ;

// *******************************************************************
// Bridge Layer code starts here
// *******************************************************************

// Turn the Fan OFF
void SetFanOff ( void )
{
	WriteAllDigitalWrapper ( 0L ) ;
}

// Turn the Fan ON
void SetFanOn ( void )
{
	WriteAllDigitalWrapper ( 0xFFFFFFFF ) ;
}

// Turn on the heaters
void SetHeatOn ( void )
{
	OutputAnalogChannelWrapper ( 2, ( long ) 0 ) ;
}

// Turn the heaters off
void SetHeatOff ( void )
{
	OutputAnalogChannelWrapper ( 2, ( long ) 255 ) ;
}

// Turn the cooling on
void SetCoolOn ( void )
{
	OutputAnalogChannelWrapper ( 1, ( long ) 0 ) ;
}

// Turn the cooling off
void SetCoolOff ( void )
{
	OutputAnalogChannelWrapper ( 1, ( long ) 255 ) ;
}

// Get ADC count value representing temp from k8055
long ReadTemperature ( void ) 
{
	long valRead  = 0 ;
	valRead = ReadAnalogChannelWrapper ( 1 ) ;
	return ( valRead ) ;
}

// 1. Connect to the hardware via USB
// 2. Turn both PWM channels to 100% (off)
// 3. Turn Fan off 
// Return >0 if opened h/w OK
long InitControllerBoard ( char* libName )
{
	long hdl = -1 ;
#if !defined (LINUX_COMPILE)
	InitWrappers ( libName ) ;
#endif

	hdl = OpenDeviceWrapper ( 0L ) ;
	printf ( "In InitControllerBoard, hdl=%ld\n", hdl ) ;

	if ( hdl >= 0 ) 
	{
		SetHeatOff ( ) ;
		SetCoolOff ( ) ;
		SetFanOff ( ) ;
	}

	return hdl ;

}

// ********************************************************************
// Application layer code starts below
// ********************************************************************

// Log data to a file
//
// Based on code by L. Spinoza
//
void logDataToFile ( unsigned long seq, char* dateTime, int setpoint, int temp, int mode )
{
	char outputLine [200] ;
	
	if ( ( seq % 3 ) == 0L )
	{
		sprintf ( outputLine, "%s,%d,%d,%d", dateTime, setpoint, temp, mode ) ;
		if ( logFilePtr ) 
		{
			fprintf ( logFilePtr, "%s\n", outputLine ) ;
		}
	}
}

// Check that cooling or heating is occurring, based on previous
// and current readings.
//
// Based on code originally developed by J. Kaluza.
//
int SystemWorkingOk (int previousReading, int currentReading, int heatCoolFan)
{
	int heat=1;
	int cool=2;
	
	//if heating is on
	if(heatCoolFan==heat && previousReading > currentReading){
		printf("previousReading, currentReading, heatCoolFan\n");
		return 0;
	}
        //if cooling is on
	if(heatCoolFan==cool && previousReading < currentReading){
		printf("previousReading, currentReading, heatCoolFan\n");
		return 0;
	}

	
	return 1;
}

// Convert ADC count to Degrees F
// Based in part on: http://www.maxim-ic.com/appnotes.cfm/an_pk/1753
//
// count is the raw count (0-255) from the ADC.
// gain is the gain of the Op-amp on the controller (2x)
// r1 is the fixed leg of the voltage divider, 47000 in this case.
//
double countToTemp ( long count, double gain, double r1 )
{
	// These are computed via calibration
	static const double a = 0.00393401767 ;
	static const double b = 0.000252561203 ;
	static const double c = -0.000000252514329 ;
	static const double d = -0.000000672358356 ;
	
	double tempK = 0.0 ;
	double tempF = 0.0 ;
	double megaOhms  = 0.0 ;
	
	// Compute thermistor resistance from voltage divider formula
	megaOhms =  ( ( ( ( 256.0 / ( ( double ) count ) ) * gain ) - 1.0 ) * r1 ) / 1000000.0 ;
	
	// Apply Steinhart-Hart to get reciprocal of temp (Kelvin)
	tempK = a + ( b * log ( megaOhms ) ) + ( c * pow ( log ( megaOhms ), 2 ) ) 
		+ ( d * pow ( log ( megaOhms ), 3 ) ) ;
		
	// Temp K	
	tempK = 1.0 / tempK ;
	
	//Convert to degrees F
	tempF = ( tempK - 273.16 ) * ( 9.0 / 5.0 ) + 32.0 ;
	
	return ( tempF ) ;
}

// Convert raw units to Degrees Farenheit (sp?)
// by "brute force" i.e. table lookup.
// This is an alternative to the direct conversion algorithm above.
// Prototype code originally developed by E. E?
float ConvertRawToTemp ( int rawValue )
{


	float tempTable[98] ;
	
	// debugging.
	//printf ( "Raw value= %3i\n", rawValue ) ;

	// Check that rawValue is within the range of
	// our table, otherwise the subscript will be
	// out of range.  Force values too low to 64
	// degrees and too high to 85 degrees.
	if ( rawValue < 101 )
		return 64;

	if  ( rawValue > 149 )
		return 85;

	// Initialize the table  (we could be clever and
	// do this only once, but this is OK for now).
	// I'm not changing all that typing... :-)
	tempTable [0] = 64;
	tempTable [1] = 64.5;
	tempTable [2] = 65;
	tempTable [3] = 65.5;
	tempTable [4] = 66;
	tempTable [5] = 68;
	tempTable [6] = 69;
	tempTable [7] = 70;
	tempTable [8] = 71;
	tempTable [9] = 72;
	tempTable [10] = 72.5;
	tempTable [11] = 73;
	tempTable [12] = 73.5;
	tempTable [13] = 74;
	tempTable [14] = 74.5;
	tempTable [15] = 75;
	tempTable [16] = 77;
	tempTable [17] = 77.5;
	tempTable [18] = 77.5;
	tempTable [19] = 78;
	tempTable [20] = 79;
	tempTable [21] = 79.5;
	tempTable [22] = 80;
	tempTable [23] = 80.5;
	tempTable [24] = 81;
	tempTable [25] = 81.5;
	tempTable [26] = 82;
	tempTable [27] = 82;
	tempTable [28] = 82;
	tempTable [29] = 82.5;
	tempTable [30] = 82.5;
	tempTable [31] = 82.5;
	tempTable [32] = 82.5;
	tempTable [33] = 82.5;
	tempTable [34] = 83;
	tempTable [35] = 83.5;
	tempTable [36] = 83.5;
	tempTable [37] = 83.5;
	tempTable [38] = 83.5;
	tempTable [39] = 83.5;
	tempTable [40] = 83.5;
	tempTable [41] = 83.5;
	tempTable [42] = 83.5;
	tempTable [43] = 84;
	tempTable [44] = 84.5;
	tempTable [45] = 84.5;
	tempTable [46] = 84.5;
	tempTable [47] = 84.5;
	tempTable [48] = 84.5;
	tempTable [49] = 84.5;
	tempTable [50] = 85;

	// Return the (converted) value from the
	/// lookup table (array).
	// adjusted this a bit to recalibrate.
	return tempTable [rawValue - 101] ;  
}


// Functions to smooth (low-pass filter) the temperature values
// and graph the results on the PC screen.
// Code originally developed by J. Tryzbiak.
//
int smoothGraph( int showGraph )
{	
	int k ;
	for (k = 0; k < 10; k++)
	{
		sum = values[k] + sum;

		//printf("%d\n", values[k]);

	}

	average = sum  / 10;	

	if ( showGraph )
	{
		/*the following printf statements print the average of the last 10 readings
		graphs them with stars*/
		printf ("%3i |", average);

		for (k = 0; k < (average / 2 ); k++)
		{
			printf("*");
		}
		printf ("\n");
	
	}
	
	return (average);
}

int graph (int currentValue, int showGraph )
{
	int result = 0 ;

	//Moves array values

	values[9] = values[8];	
	values[8] = values[7];
	values[7] = values[6];
	values[6] = values[5];
	values[5] = values[4];
	values[4] = values[3];
	values[3] = values[2];
	values[2] = values[1];
	values[1] = values[0];
	values[0] = currentValue;

	// call graphing function
	result = smoothGraph( showGraph );

	sum = 0;			

	return result ;
}

void CleanUpBeforeExit ( long hdl ) 
{
	if ( hdl >= 0 ) 
	{
		SetHeatOff ( ) ;
		SetCoolOff ( ) ;
		SetFanOff ( ) ;
		CloseDeviceWrapper ( ) ;
	}	
}

#if defined (LINUX_COMPILE)
// Based on example signal-handling code in: 
// http://users.actcom.co.il/~choo/lupg/tutorials/signals/catch-ctrl-c.c
void catchSignal ( int sig_num )
{
    /* re-set the signal handler again to catch_int, for next time */
    signal(SIGINT, catchSignal );
    gSignalCaught = 1 ;
}
#endif

// Basic bang-bang cooling algorithm
// Based on code originally developed by J. Barrett
void doCooling ( int setpoint )
{

	int previousReading = average ;
	int cooling = 2 ;
	int state = 0 ;
	int rtemp ;

	while ( ( average > setpoint ) && !gSignalCaught )
	{
		if ( !state ) 
		{
			SetCoolOn ( ) ;
			SetFanOn ( ) ;
			state = 1 ;
			printf ( "Turning cooling ON\n" ) ;\
			gMode = COOLING ;
		}

		// Read the actual room temp, in raw units
		rtemp = ( int ) countToTemp ( ReadTemperature ( ), GAIN, R1 ) ;

		average = graph ( rtemp, 1 ) ;

		// Check that system function is OK
		int workingOk = SystemWorkingOk ( previousReading, average, 
				cooling ) ;

		if ( !workingOk )
		{
			printf ( "ALARM - system failed\n" ) ;
		}

		previousReading = average ;
		logDataToFile ( ++gSeq, getFormattedDateTime ( ), setpoint, average, gMode ) ;
		sleepSeconds ( 10 ) ;

	} // end while setpoint not reached
	printf ("The fan and system are off because room temp <= desired temp.\n") ;
	SetCoolOff ( ) ;
	if ( gMode == COOLING )
	{
		startFanTimer ( gFanRunTime ) ;
	}
	gMode = IDLE ;

}

// Basic bang-bang heating algorithm
// Based on code originally developed by J. Barrett
void doHeating ( int setpoint ) 
{

	int state = 0 ;
	int rtemp ;
	
 	while ( ( average <= setpoint ) && !gSignalCaught )
 	{
		if ( !state ) 
		{
			SetHeatOn ( ) ;
			SetFanOn ( ) ;
			printf ( "Turning Heat ON\n" ) ;
			state = 1 ;
			gMode = HEATING ;
		}
 	 	// Read the actual room temp, in raw units
 	 	rtemp = ( int ) countToTemp ( ReadTemperature ( ), GAIN, R1 ) ;

 	 	average = graph ( rtemp, 1 ) ;
		logDataToFile ( ++gSeq, getFormattedDateTime ( ), setpoint, average, gMode ) ;
 	 	sleepSeconds ( 10 ) ;
 			 
 	} // end while temp not at setpoint
 	printf ("The fan and system are off because room temp >= desired temp. The system is off.\n");
 
 	SetHeatOff ( ) ;
	if ( gMode == HEATING )
	{
		startFanTimer ( gFanRunTime ) ;
	}
	gMode = IDLE ;	
}

int main(int argc, char* argv[])
{

#if !defined (LINUX_COMPILE)
	// Change this to match the location of your dll installation
	char* libName = "C:\\greed_thermo\\K8055D.dll" ;
#else
	// Not needed on Linux, because OS handles finding/loading the library
	char* libName = 0 ; 	
#endif

	char c ;
	long hdl = -1 ;
	int  rtemp = 0 ;
	int  dtemp = 0 ;
	char sys;
	int average = 0 ;
	int i ;
	char fileName[200] ;

	// Initialize the dll/library
	hdl = InitControllerBoard ( libName ) ;

	if ( hdl < 0 )
	{
		exit ( 1 ) ;
	}

#if defined (LINUX_COMPILE)	
	// setup our signal handler.  This ensures that we can
	// put the hardware in a safe state when exiting.
	
        signal ( SIGINT, catchSignal ) ;
#endif
	
	// Now prompt for settings
	printf ("Enter the state of the system. EX C = Cool, H = Heat, O = Off.\n");
        printf ("Enter the delay for the fan. EX 30 = 30 seconds ON after heat/cool OFF\n");
	printf ("Enter the desired temp (whole degrees F, from 64 to 85).\n");
        scanf ("%c %i %i", &sys, &gFanRunTime, &dtemp);
	printf ( "Running mode %c, setpoint %i, fanDelay %i\n", sys, dtemp, gFanRunTime ) ;
	
	// Setup a file and open it for logging
	strcpy ( fileName, "F" ) ;
	strcat ( fileName, getFormattedDateTime ( ) ) ;
	strcat ( fileName, ".log" ) ;
	
	logFilePtr = fopen ( fileName, "w" ) ;
	
	// "Prime" average, but don't show graph on screen
 	for ( i = 0 ; i < 10 ; i++ )
 	{
 	 	average = ( int ) countToTemp ( ReadTemperature ( ), GAIN, R1 ) ;
 	 	average = graph ( average, 0 ) ;
 	}
	
	// Thermostat logic starts here
	while ( !gSignalCaught ) 
	{

		if (sys == 'C' || sys =='c')
		{
			//gMode = COOLING ;
			doCooling ( dtemp ) ;
		} // endif mode is "cool"

		if (sys == 'H' || sys == 'h')
		{
			//gMode = HEATING ;
			doHeating ( dtemp ) ;
		}  // endif mode is "heat"
		
		// Read the actual room temp, in raw units, convert to degrees F
 	 	rtemp = ( int ) countToTemp ( ReadTemperature ( ), GAIN, R1 ) ;

		// Smooth temp value, graph on screen
 	 	average = graph ( rtemp, 1 ) ;
		
		// Log the run to a file for later analysis of performance.
		logDataToFile ( ++gSeq, getFormattedDateTime ( ), dtemp, average, IDLE ) ;
		
		// Kill some time
		sleepSeconds ( 10 ) ;
		
		// If we're running the fan after heating or cooling
		// has ceased, see if time to turn it off.
		if ( runFanTimer ( ) )
		{
			SetFanOff ( ) ;
		}
	}
	
	
	printf ( "Press x to exit\n" ) ;
	fflush ( stdin ) ;	
	scanf ( "%c", &c ) ;

	if ( logFilePtr )
	{
		fclose ( logFilePtr ) ;
	}
	CleanUpBeforeExit ( hdl ) ;	

	return 0;
}

